/*
 * grader-io - tty.h
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 * based on linux/tty.h from Linux v4.8
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __TTY_H
#define __TTY_H

struct tty_struct;

struct tty_operations {
	int (*write)(struct tty_struct *tty, const unsigned char *buf,
			int count);
	int (*write_room)(struct tty_struct *tty);
};

struct tty_ldisc_ops {
	void (*receive_buf)(struct tty_struct *tty, const unsigned char *cp,
			char *fp, int count);
};

struct tty_struct {
	const struct tty_operations *ops;
	const struct tty_ldisc_ops *ldisc_ops;
	void *disc_data;
};

static inline void tty_receive_buf(struct tty_struct *tty,
		const unsigned char *cp, char *fp, int count)
{
	tty->ldisc_ops->receive_buf(tty, cp, fp, count);
}

void tty_set_ldisc(struct tty_struct *tty, const struct tty_ldisc_ops *ops,
		void *disc_data);

#define TTY_NORMAL	0
#define TTY_BREAK	1
#define TTY_FRAME	2
#define TTY_PARITY	3
#define TTY_OVERRUN	4

#endif
