/*
 * grader-io - gpio.c
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <asm_io.h>
#include <lpc176x.h>
#include <gpio.h>

static u32 gpio_read(unsigned int bank, unsigned int reg)
{
	return readl((u8 *) GPIO_BASE + (bank << 5) + reg);
}

static void gpio_write(u32 x, unsigned int bank, unsigned int reg)
{
	writel(x, (u8 *) GPIO_BASE + (bank << 5) + reg);
}

int __gpio_get_value(unsigned int gpio)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (unlikely(gpio >= NUM_GPIO))
		return -EINVAL;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = gpio_read(bank, GPIO_FIOPIN);
	return (x & (1 << bit)) != 0;
}

void __gpio_set_value(unsigned int gpio, int value)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (unlikely(gpio >= NUM_GPIO))
		return;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = 1 << bit;
	if (value)
		gpio_write(x, bank, GPIO_FIOSET);
	else
		gpio_write(x, bank, GPIO_FIOCLR);
}

int gpio_direction_input(unsigned int gpio)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (gpio >= NUM_GPIO)
		return -EINVAL;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = gpio_read(bank, GPIO_FIOMASK);
	x &= ~(1 << bit);
	gpio_write(x, bank, GPIO_FIOMASK);

	x = gpio_read(bank, GPIO_FIODIR);
	x &= ~(1 << bit);
	gpio_write(x, bank, GPIO_FIODIR);
	return 0;
}

int gpio_direction_output(unsigned int gpio, int value)
{
	unsigned int bank;
	unsigned int bit;
	u32 x;

	if (gpio >= NUM_GPIO)
		return -EINVAL;

	bank = gpio >> 5;
	bit = gpio & 0x1f;

	x = gpio_read(bank, GPIO_FIOMASK);
	x &= ~(1 << bit);
	gpio_write(x, bank, GPIO_FIOMASK);
	__gpio_set_value(gpio, value);

	x = gpio_read(bank, GPIO_FIODIR);
	x |= 1 << bit;
	gpio_write(x, bank, GPIO_FIODIR);
	return 0;
}
