/*
 * grader-io - posix-gpio.c
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <gpio.h>
#include <stdio.h>
#include <time.h>

int __gpio_get_value(unsigned int gpio)
{
	/* node address 0 */
	if (gpio >= 4 && gpio < 8)
		return 1;
	if (gpio == 0x38) {
		/* SYNC, simulate 1 pulse per 8 secs */
		struct timespec ts;

		clock_gettime(CLOCK_REALTIME, &ts);
		return (ts.tv_sec & 7) == 0 && (ts.tv_nsec < 100000000);
	}
	if (gpio == 0x39) {
		/* PULSE, simulate 1 pulse per second */
		struct timespec ts;

		clock_gettime(CLOCK_REALTIME, &ts);
		return ts.tv_nsec < 100000000;
	}
	return 0;
}

void __gpio_set_value(unsigned int gpio, int value)
{
//	fprintf(stderr, "gpio: PIN%d = %d\n", gpio, value != 0);
}

int gpio_direction_input(unsigned int gpio)
{
	fprintf(stderr, "gpio: PIN%d is now an input\n", gpio);
	return 0;
}

int gpio_direction_output(unsigned int gpio, int value)
{
	fprintf(stderr, "gpio: PIN%d is now an output, initial value %d\n",
			gpio, value != 0);
	return 0;
}
